% Differential amplifier script by Ethan Crain and Michael Perrott
% from the High Speed Circuits and Systems Group at MIT (part of MTL)
% http://www-mtl.mit.edu/research/perrottgroup/tools.html
% Copyright 2004 by Ethan Crain and Michael Perrott
%
% Script for generating device size and biasing for resistively
% loaded differential pair.  Inputs are output voltage swing (Vsw),
% DC gain (Av), power (Ibias), scaling factor (Scale) which
% represents the ratio of widths for the loading stage and the stage
% under test, fixed load capacitance (Cfix)model corner and the
% simulation characterization file name (file).  The script returns
% the input pair width (assuming the length is the same as for the
% characterized device), bias current, load resistance (RlE), bandwidth
% (FbwoE) and calculated gain (AvoeE).
%
% Format for specifying corner:  <model corner><temp>
%        choices: 's-40','s25','s85','t-40','t25','t85','f-40','f25','f85'
%        This can be modified in the Hspice characterization netlist
%
% [WnE,RlE,FbwoE]=diffampPWR(Vsw,Av,Ibias,Scale,Cfix,corner)
% Alternatively...
% [WnE,RlE,FbwoE,IbiasE,VswoE,AvoE]=diffampPWR(Vsw,Av,Ibias,Scale,Cfix,corner)

function [WnE,RlE,FbwoE,IbiasE,VswoE,AvoE]=diffampPWR(Vsw,Av,Ibias,Scale,Cfix,corner)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Variables that need to be updated for specific process
%% Note:  The default values listed are not derived from any actual
%%        model file or process.  The numbers are completely arbitrary
%%        and do not reflect any specific or general knowledge of 
%%        proprietary process information.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Hdin = 0.5e-6;            % minimum s/d extension w/ sharing (m)
Hdout = 0.5e-6;           % minimum s/d extension w/out sharing (m)
Rsh = 200;                % sheet rho of resistor (ohms/sq)
Lmin = 0.18e-6;           % use minimum L devices (m) (same as char file
CRp = 0.5*100e-6;         % 1/2 of capacitance of resistor (F/um^2)
Idenmax = 0.5e3;          % maximum current density for resistor (A/m)
MinSq = 1;                % Minimum number of squares for poly resistors
%%Cfix = 15*10^-15;       % Assume fixed wiring cap. (F)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% End of variables that need to be updated 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch corner
    case 't-40'
        file1 = 'spicefiles/test.sw4';
        file2 = 'spicefiles/test.ac4';
    case 't25'
        file1 = 'spicefiles/test.sw5';
        file2 = 'spicefiles/test.ac5';
    case 't85'
        file1 = 'spicefiles/test.sw6';
        file2 = 'spicefiles/test.ac6';
    case 's-40'
        file1 = 'spicefiles/test.sw1';
        file2 = 'spicefiles/test.ac1';
    case 's25'
        file1 = 'spicefiles/test.sw2';
        file2 = 'spicefiles/test.ac2';
    case 's85'
        file1 = 'spicefiles/test.sw3';
        file2 = 'spicefiles/test.ac3';
    case 'f-40'
        file1 = 'spicefiles/test.sw7';
        file2 = 'spicefiles/test.ac7';
    case 'f25'
        file1 = 'spicefiles/test.sw8';
        file2 = 'spicefiles/test.ac8';
    case 'f85'
        file1 = 'spicefiles/test.sw9';
        file2 = 'spicefiles/test.ac9';
end;

% Empirical solution for device width - need to call hspice
% and extract typical gm/W value for the given Av and Vsw
% Alternately, load characterization file

%hspsim                             % file is test.par
x = loadsig(file1);                 % load hspice dc data
gmSIM = evalsig(x,'lx7_m0');        % simulated gm
gdsSIM = evalsig(x,'lx8_m0');       % simulated gds
IdSIM = evalsig(x,'i_m0');          % simulated Id
wSIM = 2*evalsig(x,'w');            % width from char
y = loadsig(file2);                 % load hspice ac data
cgSIM = 2*evalsig(y,'lx18_m0');     % cg for load device
cgdSIM = -2*evalsig(y,'lx19_m0');   % cgd for load device
cgsSIM = -2*evalsig(y,'lx20_m0');   % cgs for load device
cgbSIM = -2*evalsig(y,'lx21_m0');   % cgb for load device
cdbSIM = -2*evalsig(y,'lx22_m0');   % cdb for driving device

% Determine the appropriate current density and
% transconductance density from characterization
% data based on desired gain and voltage swing.

k = length(wSIM);
while (gmSIM(k)/wSIM(k))>(2*Av/Vsw*(IdSIM(k)/wSIM(k))+Av*gdsSIM(k)/wSIM(k))
   k=k-1;
end;

gmDen = gmSIM(k+1)/wSIM(k+1);         % empirical gm/w
gdsDen = gdsSIM(k+1)/wSIM(k+1);       % empirical gds/w
iDen = IdSIM(k+1)/wSIM(k+1);          % empirical ibias/w
cgDen = cgSIM(k+1)/wSIM(k+1);         % empirical cg/w
cgdDen = cgdSIM(k+1)/wSIM(k+1);       % empirical cgd/w
cgsDen = cgsSIM(k+1)/wSIM(k+1);       % empirical cgs/w
cgbDen = cgbSIM(k+1)/wSIM(k+1);       % empirical cgb/w
cdbDen = cdbSIM(k+1)/wSIM(k+1);       % empirical cds/w

% Determine the circuit parameters based on the process
% characterization data, design specifications and the script

% These equations are for inputs - Vsw, Av, Ibias
IbiasE = Ibias;
RlE = Vsw/IbiasE;
VswoE = IbiasE*RlE;
WnE = 0.5*IbiasE/iDen;
AvoE = gmDen*WnE*RlE/(1+gdsDen*WnE*RlE);
%% To ingnore Miller effect use this calculation for load device capacitance
%Cvar = WnE*(cdbDen+Scale*cgDen);
%% To account for Miller effect, use this calculation
Cvar = WnE*(cdbDen+Scale*(cgsDen+cgbDen+(1+AvoE)*cgdDen));


% Need to guarantee that Nsq >= MinSq

% This code calculates resistor size based on satisfying
% minimum number of squares (Nsq) and current density
% requirements based on design rule recommendations for
% device matching.

% Nsq = VswoE/(IbiasE*Rsh);
% Width = 2e-6;
% if ((Nsq >= MinSq)&&(IbiasE/Width <= Idenmax))
%     Length = Width*Nsq;
% else if ((Nsq >= MinSq)&&(IbiasE/Width > Idenmax))
%         Width = IbiasE/Idenmax;
%         Length = Nsq*Width;
%     else if ((Nsq < MinSq)&&(IbiasE/Width <= Idenmax))
%                Width = ceil(MinSq/Nsq)*2e-6;
%                Length = Nsq*Width*ceil(MinSq/Nsq);
%         else if ((Nsq < MinSq)&&(IbiasE/Width > Idenmax))
%                 Width = max(ceil(MinSq/Nsq)*2e-6, IbiasE/Idenmax);
%                 Length = Nsq*Width*ceil(MinSq/Nsq);
%             end
%         end
%     end
% end

% This code calculates resistor size based on satisfying
% current density rules only and optimizes the size for
% speed.  The total area is kept larger than 10um^2.

Nsq = VswoE/(IbiasE*Rsh);
Width = max(2e-6,IbiasE/Idenmax);
Length = Nsq*Width;
if (Width*Length < 10e-12)
    ratio = sqrt(10e-12/Width/Length);
    Width = Width*ratio;
    Length = Length*ratio;
end

FbwoE = (2*pi*RlE/(1+gdsDen*WnE*RlE)*(Cfix+Cvar+CRp*Length*Width))^-1;

